@@ -5,6 +5,7 @@
 #include "timers.h"
 #include "device.h"
 #include "config.h"
+#include "ratelimiter.h"
 #include "peer.h"
 #include "uapi.h"
 #include "messages.h"
@@ -21,23 +22,13 @@
 #include <net/rtnetlink.h>
 #include <net/ip_tunnels.h>
 #include <net/addrconf.h>
-#if IS_ENABLED(CONFIG_NF_CONNTRACK)
-#include <net/netfilter/nf_conntrack.h>
-#include <net/netfilter/nf_nat_core.h>
-#endif
 
-static int open_peer(struct wireguard_peer *peer, void *data)
-{
-	timers_init_peer(peer);
-	packet_send_queue(peer);
-	if (peer->persistent_keepalive_interval)
-		packet_send_keepalive(peer);
-	return 0;
-}
+static LIST_HEAD(device_list);
 
 static int open(struct net_device *dev)
 {
 	int ret;
+	struct wireguard_peer *peer, *temp;
 	struct wireguard_device *wg = netdev_priv(dev);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 17, 0)
 	struct inet6_dev *dev_v6 = __in6_dev_get(dev);
@@ -64,92 +55,80 @@ static int open(struct net_device *dev)
 	ret = socket_init(wg);
 	if (ret < 0)
 		return ret;
-	peer_for_each(wg, open_peer, NULL);
-	return 0;
-}
-
-static int clear_noise_peer(struct wireguard_peer *peer, void *data)
-{
-	noise_handshake_clear(&peer->handshake);
-	noise_keypairs_clear(&peer->keypairs);
-	if (peer->timers_enabled)
-		del_timer(&peer->timer_kill_ephemerals);
+	peer_for_each (wg, peer, temp, true) {
+		timers_init_peer(peer);
+		packet_send_queue(peer);
+		if (peer->persistent_keepalive_interval)
+			packet_send_keepalive(peer);
+	}
 	return 0;
 }
 
 #ifdef CONFIG_PM_SLEEP
 static int suspending_clear_noise_peers(struct notifier_block *nb, unsigned long action, void *data)
 {
-	struct wireguard_device *wg = container_of(nb, struct wireguard_device, clear_peers_on_suspend);
-	if (action == PM_HIBERNATION_PREPARE || action == PM_SUSPEND_PREPARE) {
-		peer_for_each(wg, clear_noise_peer, NULL);
-		rcu_barrier();
+	struct wireguard_device *wg;
+	struct wireguard_peer *peer, *temp;
+
+	if (action != PM_HIBERNATION_PREPARE && action != PM_SUSPEND_PREPARE)
+		return 0;
+
+	rtnl_lock();
+	list_for_each_entry (wg, &device_list, device_list) {
+		peer_for_each (wg, peer, temp, true) {
+			noise_handshake_clear(&peer->handshake);
+			noise_keypairs_clear(&peer->keypairs);
+			if (peer->timers_enabled)
+				del_timer(&peer->timer_zero_key_material);
+		}
 	}
-	return 0;
-}
-#endif
+	rtnl_unlock();
+	rcu_barrier_bh();
 
-static int stop_peer(struct wireguard_peer *peer, void *data)
-{
-	timers_uninit_peer(peer);
-	clear_noise_peer(peer, data);
 	return 0;
 }
+static struct notifier_block clear_peers_on_suspend = { .notifier_call = suspending_clear_noise_peers };
+#endif
 
 static int stop(struct net_device *dev)
 {
 	struct wireguard_device *wg = netdev_priv(dev);
-	peer_for_each(wg, stop_peer, NULL);
+	struct wireguard_peer *peer, *temp;
+	peer_for_each (wg, peer, temp, true) {
+		timers_uninit_peer(peer);
+		noise_handshake_clear(&peer->handshake);
+		noise_keypairs_clear(&peer->keypairs);
+		if (peer->timers_enabled)
+			del_timer(&peer->timer_zero_key_material);
+	}
 	skb_queue_purge(&wg->incoming_handshakes);
 	socket_uninit(wg);
 	return 0;
 }
 
-static void skb_unsendable(struct sk_buff *skb, struct net_device *dev)
-{
-#if IS_ENABLED(CONFIG_NF_CONNTRACK)
-	/* This conntrack stuff is because the rate limiting needs to be applied
-	 * to the original src IP, so we have to restore saddr in the IP header.
-	 * It's not needed if conntracking isn't in the kernel, because in that
-	 * case the saddr wouldn't be NAT-transformed anyway. */
-	enum ip_conntrack_info ctinfo;
-	struct nf_conn *ct = nf_ct_get(skb, &ctinfo);
-#endif
-	++dev->stats.tx_errors;
-
-	if (skb->len >= sizeof(struct iphdr) && ip_hdr(skb)->version == 4) {
-#if IS_ENABLED(CONFIG_NF_CONNTRACK)
-		if (ct)
-			ip_hdr(skb)->saddr = ct->tuplehash[0].tuple.src.u3.ip;
-#endif
-		icmp_send(skb, ICMP_DEST_UNREACH, ICMP_HOST_UNREACH, 0);
-	} else if (skb->len >= sizeof(struct ipv6hdr) && ip_hdr(skb)->version == 6) {
-#if IS_ENABLED(CONFIG_NF_CONNTRACK)
-		if (ct)
-			ipv6_hdr(skb)->saddr = ct->tuplehash[0].tuple.src.u3.in6;
-#endif
-		icmpv6_send(skb, ICMPV6_DEST_UNREACH, ICMPV6_ADDR_UNREACH, 0);
-	}
-	kfree_skb(skb);
-}
-
 static netdev_tx_t xmit(struct sk_buff *skb, struct net_device *dev)
 {
 	struct wireguard_device *wg = netdev_priv(dev);
 	struct wireguard_peer *peer;
+	struct sk_buff *next;
 	int ret;
 
 	if (unlikely(dev_recursion_level() > 4)) {
 		ret = -ELOOP;
-		net_dbg_ratelimited("Routing loop detected\n");
-		skb_unsendable(skb, dev);
+		net_dbg_ratelimited("%s: Routing loop detected\n", dev->name);
+		goto err;
+	}
+
+	if (unlikely(skb_examine_untrusted_ip_hdr(skb) != skb->protocol)) {
+		ret = -EPROTONOSUPPORT;
+		net_dbg_ratelimited("%s: Invalid IP packet\n", dev->name);
 		goto err;
 	}
 
 	peer = routing_table_lookup_dst(&wg->peer_routing_table, skb);
 	if (unlikely(!peer)) {
 		ret = -ENOKEY;
-		net_dbg_skb_ratelimited("No peer is configured for %pISc\n", skb);
+		net_dbg_skb_ratelimited("%s: No peer is configured for %pISc\n", dev->name, skb);
 		goto err;
 	}
 
@@ -157,8 +136,8 @@ static netdev_tx_t xmit(struct sk_buff *skb, struct net_device *dev)
 	ret = peer->endpoint.addr.sa_family != AF_INET && peer->endpoint.addr.sa_family != AF_INET6;
 	read_unlock_bh(&peer->endpoint_lock);
 	if (unlikely(ret)) {
-		ret = -EHOSTUNREACH;
-		net_dbg_ratelimited("No valid endpoint has been configured or discovered for peer %Lu\n", peer->internal_id);
+		ret = -EDESTADDRREQ;
+		net_dbg_ratelimited("%s: No valid endpoint has been configured or discovered for peer %Lu\n", dev->name, peer->internal_id);
 		goto err_peer;
 	}
 
@@ -178,8 +157,8 @@ static netdev_tx_t xmit(struct sk_buff *skb, struct net_device *dev)
 		dev_kfree_skb(skb);
 		skb = segs;
 	}
-	while (skb) {
-		struct sk_buff *next = skb->next;
+	do {
+		next = skb->next;
 		skb->next = skb->prev = NULL;
 
 		skb = skb_share_check(skb, GFP_ATOMIC);
@@ -191,8 +170,7 @@ static netdev_tx_t xmit(struct sk_buff *skb, struct net_device *dev)
 		skb_dst_drop(skb);
 
 		skb_queue_tail(&peer->tx_packet_queue, skb);
-		skb = next;
-	}
+	} while ((skb = next) != NULL);
 
 	packet_send_queue(peer);
 	peer_put(peer);
@@ -201,11 +179,15 @@ static netdev_tx_t xmit(struct sk_buff *skb, struct net_device *dev)
 err_peer:
 	peer_put(peer);
 err:
-	skb_unsendable(skb, dev);
+	++dev->stats.tx_errors;
+	if (skb->protocol == htons(ETH_P_IP))
+		icmp_send(skb, ICMP_DEST_UNREACH, ICMP_HOST_UNREACH, 0);
+	else if (skb->protocol == htons(ETH_P_IPV6))
+		icmpv6_send(skb, ICMPV6_DEST_UNREACH, ICMPV6_ADDR_UNREACH, 0);
+	kfree_skb(skb);
 	return ret;
 }
 
-
 static int ioctl(struct net_device *dev, struct ifreq *ifr, int cmd)
 {
 	struct wireguard_device *wg = netdev_priv(dev);
@@ -234,29 +216,30 @@ static void destruct(struct net_device *dev)
 {
 	struct wireguard_device *wg = netdev_priv(dev);
 
+	rtnl_lock();
+	list_del(&wg->device_list);
+	rtnl_unlock();
 	mutex_lock(&wg->device_update_lock);
 	peer_remove_all(wg);
 	wg->incoming_port = 0;
-	destroy_workqueue(wg->handshake_wq);
+	destroy_workqueue(wg->incoming_handshake_wq);
+	destroy_workqueue(wg->peer_wq);
 #ifdef CONFIG_WIREGUARD_PARALLEL
 	padata_free(wg->encrypt_pd);
 	padata_free(wg->decrypt_pd);
 	destroy_workqueue(wg->crypt_wq);
 #endif
 	routing_table_free(&wg->peer_routing_table);
+	ratelimiter_uninit();
 	memzero_explicit(&wg->static_identity, sizeof(struct noise_static_identity));
 	skb_queue_purge(&wg->incoming_handshakes);
 	socket_uninit(wg);
-	cookie_checker_uninit(&wg->cookie_checker);
-#ifdef CONFIG_PM_SLEEP
-	unregister_pm_notifier(&wg->clear_peers_on_suspend);
-#endif
 	mutex_unlock(&wg->device_update_lock);
 	free_percpu(dev->tstats);
-
+	free_percpu(wg->incoming_handshakes_worker);
 	put_net(wg->creating_net);
 
-	pr_debug("Device %s has been deleted\n", dev->name);
+	pr_debug("%s: Interface deleted\n", dev->name);
 	free_netdev(dev);
 }
 
@@ -266,7 +249,6 @@ static void setup(struct net_device *dev)
 	enum { WG_NETDEV_FEATURES = NETIF_F_HW_CSUM | NETIF_F_RXCSUM | NETIF_F_SG | NETIF_F_GSO | NETIF_F_GSO_SOFTWARE | NETIF_F_HIGHDMA };
 
 	dev->netdev_ops = &netdev_ops;
-	dev->destructor = destruct;
 	dev->hard_header_len = 0;
 	dev->addr_len = 0;
 	dev->needed_headroom = DATA_PACKET_HEAD_ROOM;
@@ -288,11 +270,12 @@ static void setup(struct net_device *dev)
 	netif_keep_dst(dev);
 
 	memset(wg, 0, sizeof(struct wireguard_device));
+	wg->dev = dev;
 }
 
-static int newlink(struct net *src_net, struct net_device *dev, struct nlattr *tb[], struct nlattr *data[])
+static int newlink(struct net *src_net, struct net_device *dev, struct nlattr *tb[], struct nlattr *data[], struct netlink_ext_ack *extack)
 {
-	int ret = -ENOMEM;
+	int ret = -ENOMEM, cpu;
 	struct wireguard_device *wg = netdev_priv(dev);
 
 	wg->creating_net = get_net(src_net);
@@ -300,71 +283,82 @@ static int newlink(struct net *src_net, struct net_device *dev, struct nlattr *t
 	mutex_init(&wg->socket_update_lock);
 	mutex_init(&wg->device_update_lock);
 	skb_queue_head_init(&wg->incoming_handshakes);
-	INIT_WORK(&wg->incoming_handshakes_work, packet_process_queued_handshake_packets);
 	pubkey_hashtable_init(&wg->peer_hashtable);
 	index_hashtable_init(&wg->index_hashtable);
 	routing_table_init(&wg->peer_routing_table);
+	cookie_checker_init(&wg->cookie_checker, wg);
 	INIT_LIST_HEAD(&wg->peer_list);
 
 	dev->tstats = netdev_alloc_pcpu_stats(struct pcpu_sw_netstats);
 	if (!dev->tstats)
 		goto error_1;
 
-	wg->handshake_wq = alloc_workqueue("wg-kex-%s", WQ_UNBOUND | WQ_FREEZABLE, 0, dev->name);
-	if (!wg->handshake_wq)
+	wg->incoming_handshakes_worker = alloc_percpu(struct handshake_worker);
+	if (!wg->incoming_handshakes_worker)
 		goto error_2;
+	for_each_possible_cpu (cpu) {
+		per_cpu_ptr(wg->incoming_handshakes_worker, cpu)->wg = wg;
+		INIT_WORK(&per_cpu_ptr(wg->incoming_handshakes_worker, cpu)->work, packet_process_queued_handshake_packets);
+	}
+	atomic_set(&wg->incoming_handshake_seqnr, 0);
+
+	wg->incoming_handshake_wq = alloc_workqueue("wg-kex-%s", WQ_CPU_INTENSIVE | WQ_FREEZABLE, 0, dev->name);
+	if (!wg->incoming_handshake_wq)
+		goto error_3;
+
+	wg->peer_wq = alloc_workqueue("wg-kex-%s", WQ_UNBOUND | WQ_FREEZABLE, 0, dev->name);
+	if (!wg->peer_wq)
+		goto error_4;
 
 #ifdef CONFIG_WIREGUARD_PARALLEL
 	wg->crypt_wq = alloc_workqueue("wg-crypt-%s", WQ_CPU_INTENSIVE | WQ_MEM_RECLAIM, 2, dev->name);
 	if (!wg->crypt_wq)
-		goto error_3;
+		goto error_5;
 
 	wg->encrypt_pd = padata_alloc_possible(wg->crypt_wq);
 	if (!wg->encrypt_pd)
-		goto error_4;
+		goto error_6;
 	padata_start(wg->encrypt_pd);
 
 	wg->decrypt_pd = padata_alloc_possible(wg->crypt_wq);
 	if (!wg->decrypt_pd)
-		goto error_5;
+		goto error_7;
 	padata_start(wg->decrypt_pd);
 #endif
 
-	ret = cookie_checker_init(&wg->cookie_checker, wg);
-	if (ret < 0)
-		goto error_6;
-
-#ifdef CONFIG_PM_SLEEP
-	wg->clear_peers_on_suspend.notifier_call = suspending_clear_noise_peers;
-	ret = register_pm_notifier(&wg->clear_peers_on_suspend);
-	if (ret < 0)
-		goto error_7;
-#endif
-
-	ret = register_netdevice(dev);
+	ret = ratelimiter_init();
 	if (ret < 0)
 		goto error_8;
 
-	pr_debug("Device %s has been created\n", dev->name);
+	ret = register_netdevice(dev);
+	if (ret < 0)
+		goto error_9;
 
-	return 0;
+	list_add(&wg->device_list, &device_list);
 
+	/* We wait until the end to assign priv_destructor, so that register_netdevice doesn't
+	 * call it for us if it fails. */
+	dev->priv_destructor = destruct;
+
+	pr_debug("%s: Interface created\n", dev->name);
+	return ret;
+
+error_9:
+	ratelimiter_uninit();
 error_8:
-#ifdef CONFIG_PM_SLEEP
-	unregister_pm_notifier(&wg->clear_peers_on_suspend);
-error_7:
-#endif
-	cookie_checker_uninit(&wg->cookie_checker);
-error_6:
 #ifdef CONFIG_WIREGUARD_PARALLEL
 	padata_free(wg->decrypt_pd);
-error_5:
+error_7:
 	padata_free(wg->encrypt_pd);
-error_4:
+error_6:
 	destroy_workqueue(wg->crypt_wq);
-error_3:
+error_5:
 #endif
-	destroy_workqueue(wg->handshake_wq);
+	destroy_workqueue(wg->peer_wq);
+error_4:
+	destroy_workqueue(wg->incoming_handshake_wq);
+error_3:
+	free_percpu(wg->incoming_handshakes_worker);
 error_2:
 	free_percpu(dev->tstats);
 error_1:
@@ -379,13 +373,21 @@ static struct rtnl_link_ops link_ops __read_mostly = {
 	.newlink		= newlink,
 };
 
-int device_init(void)
+int __init device_init(void)
 {
+#ifdef CONFIG_PM_SLEEP
+	int ret = register_pm_notifier(&clear_peers_on_suspend);
+	if (ret)
+		return ret;
+#endif
 	return rtnl_link_register(&link_ops);
 }
 
-void device_uninit(void)
+void __exit device_uninit(void)
 {
 	rtnl_link_unregister(&link_ops);
-	rcu_barrier();
+#ifdef CONFIG_PM_SLEEP
+	unregister_pm_notifier(&clear_peers_on_suspend);
+#endif
+	rcu_barrier_bh();
 }
